<?php
declare(strict_types=1);

namespace Inti\App\Runner\Container;

use Inti\Container\Exception\MissingDependencyException;
use Laminas\HttpHandlerRunner\Emitter\EmitterInterface;
use Laminas\HttpHandlerRunner\RequestHandlerRunner;
use Laminas\Stratigility\Middleware\ErrorResponseGenerator;
use Psr\Container\ContainerInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\RequestHandlerInterface;

/**
 * Class RequestHandlerRunnerFactory
 *
 * @package Inti\App\Runner\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-app-runner
 */
final class RequestHandlerRunnerFactory
{
    private array $dependencies = [
        EmitterInterface::class,
        ErrorResponseGenerator::class,
        RequestHandlerInterface::class,
        ServerRequestInterface::class,
    ];

    public function __invoke(ContainerInterface $container): RequestHandlerRunner
    {
        foreach ($this->dependencies as $dependency) {
            if (!$container->has($dependency)) {
                throw MissingDependencyException::dependencyForService(
                    $dependency,
                    RequestHandlerRunner::class
                );
            }
        }

        return new RequestHandlerRunner(
            $container->get(RequestHandlerInterface::class),
            $container->get(EmitterInterface::class),
            $container->get(ServerRequestInterface::class),
            $container->get(ErrorResponseGenerator::class)
        );
    }
}
